/* ---------------------------------------------------------------------------
** This software is part of 16FUSB project.
**
** 16FUSBDioAppDemo.cpp
** Main application file.
**
** Author: Emanuel Paz <efspaz@gmail.com>
** -------------------------------------------------------------------------*/
#include "stdafx.h"
#include "16FUSBDioSampleApp.h"
#include "16FUSB_Dio.h"

#define MAX_LOADSTRING 100

// Global Variables:
HINSTANCE hInst;								// current instance
TCHAR szTitle[MAX_LOADSTRING];					// The title bar text
TCHAR szWindowClass[MAX_LOADSTRING];			// the main window class name

static HICON hIcon;

INT_PTR CALLBACK	Main(HWND, UINT, WPARAM, LPARAM);
INT_PTR CALLBACK	About(HWND, UINT, WPARAM, LPARAM);

void	tryReestablishe(HWND);

int APIENTRY _tWinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPTSTR lpCmdLine, int nCmdShow)
{
	hInst = hInstance;
	return DialogBox(hInst, MAKEINTRESOURCE(IDD_MAINWND), NULL, Main);
}


// Message handler for about box.
INT_PTR CALLBACK Main(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
	UNREFERENCED_PARAMETER(lParam);
	switch (message)
	{
	case WM_INITDIALOG:
		hIcon = (HICON)LoadImage(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_SMALL), IMAGE_ICON, 32, 32, 0);
		SendMessage(hDlg, WM_SETICON, ICON_BIG, (LPARAM)hIcon);
		CheckDlgButton(hDlg, IDC_WBT, BST_CHECKED);
		CheckDlgButton(hDlg, IDC_RDBT, BST_CHECKED);

		if(init16FUSB() == -1) {
			MessageBox(NULL, L"Device not found!", L"16FUSB", NULL);
			EndDialog(hDlg, LOWORD(wParam));
		}
		return (INT_PTR)TRUE;

	case WM_COMMAND:
		if (LOWORD(wParam) == IDM_EXIT || LOWORD(wParam) == IDCANCEL) {
			close16FUSB();
			EndDialog(hDlg, LOWORD(wParam));
			return (INT_PTR)TRUE;
		} else if(LOWORD(wParam) == IDM_ABOUT) {			// About Dialog
			DialogBox(hInst, MAKEINTRESOURCE(ABOUT_DIALOG), hDlg, About);
			break;		
		} else if(LOWORD(wParam) == IDC_RDBUTTON) {			// Read button pressed
			
			char buffer[1] = {0};

			if(IsDlgButtonChecked(hDlg, IDC_RDBT) == BST_CHECKED) {				
				if(readByte(buffer) > 0) {
					CheckDlgButton(hDlg, IDC_RDIOBIT0, buffer[0] & 1 ? BST_CHECKED: BST_UNCHECKED);
					CheckDlgButton(hDlg, IDC_RDIOBIT1, buffer[0] & 2 ? BST_CHECKED: BST_UNCHECKED);
					CheckDlgButton(hDlg, IDC_RDIOBIT2, buffer[0] & 4 ? BST_CHECKED: BST_UNCHECKED);
					CheckDlgButton(hDlg, IDC_RDIOBIT3, buffer[0] & 8 ? BST_CHECKED: BST_UNCHECKED);
					CheckDlgButton(hDlg, IDC_RDIOBIT4, buffer[0] & 16 ? BST_CHECKED: BST_UNCHECKED);
					CheckDlgButton(hDlg, IDC_RDIOBIT5, buffer[0] & 32 ? BST_CHECKED: BST_UNCHECKED);
					CheckDlgButton(hDlg, IDC_RDIOBIT6, buffer[0] & 64 ? BST_CHECKED: BST_UNCHECKED);
					CheckDlgButton(hDlg, IDC_RDIOBIT7, buffer[0] & 128 ? BST_CHECKED: BST_UNCHECKED);
				} else {
					tryReestablishe(hDlg);
				}
			} else if(IsDlgButtonChecked(hDlg, IDC_RDLN) == BST_CHECKED) {
				if(readLowNibble(buffer) > 0) {
					CheckDlgButton(hDlg, IDC_RDIOBIT0, buffer[0] & 1 ? BST_CHECKED: BST_UNCHECKED);
					CheckDlgButton(hDlg, IDC_RDIOBIT1, buffer[0] & 2 ? BST_CHECKED: BST_UNCHECKED);
					CheckDlgButton(hDlg, IDC_RDIOBIT2, buffer[0] & 4 ? BST_CHECKED: BST_UNCHECKED);
					CheckDlgButton(hDlg, IDC_RDIOBIT3, buffer[0] & 8 ? BST_CHECKED: BST_UNCHECKED);					
				} else {
					tryReestablishe(hDlg);
				}
			} else if(IsDlgButtonChecked(hDlg, IDC_RDHN) == BST_CHECKED) {				
				if(readHighNibble(buffer) > 0) {
					CheckDlgButton(hDlg, IDC_RDIOBIT4, buffer[0] & 1 ? BST_CHECKED: BST_UNCHECKED);
					CheckDlgButton(hDlg, IDC_RDIOBIT5, buffer[0] & 2 ? BST_CHECKED: BST_UNCHECKED);
					CheckDlgButton(hDlg, IDC_RDIOBIT6, buffer[0] & 4 ? BST_CHECKED: BST_UNCHECKED);
					CheckDlgButton(hDlg, IDC_RDIOBIT7, buffer[0] & 8 ? BST_CHECKED: BST_UNCHECKED);					
				} else {
					tryReestablishe(hDlg);
				}

			} else if(IsDlgButtonChecked(hDlg, IDC_RDCT) == BST_CHECKED) {
				if(readCtrl(buffer) > 0) {
					CheckDlgButton(hDlg, IDC_RDCTBIT0, buffer[0] & 1 ? BST_CHECKED: BST_UNCHECKED);
					CheckDlgButton(hDlg, IDC_RDCTBIT1, buffer[0] & 2 ? BST_CHECKED: BST_UNCHECKED);
				} else {
					tryReestablishe(hDlg);
				}
			} else if(IsDlgButtonChecked(hDlg, IDC_RDST) == BST_CHECKED) {
				if(readStatus(buffer) > 0) {
					CheckDlgButton(hDlg, IDC_RDSTBIT0, buffer[0] & 1 ? BST_CHECKED: BST_UNCHECKED);
				} else {
					tryReestablishe(hDlg);
				}
			}
		} else if(LOWORD(wParam) == IDC_WBUTTON) {			// Write button pressed

			int byteToSend = 0;

			if(IsDlgButtonChecked(hDlg, IDC_WBT) == BST_CHECKED) {
				byteToSend |= (IsDlgButtonChecked(hDlg, IDC_WIOBIT0) == BST_CHECKED) ? 1: 0;
				byteToSend |= (IsDlgButtonChecked(hDlg, IDC_WIOBIT1) == BST_CHECKED) ? 2: 0;
				byteToSend |= (IsDlgButtonChecked(hDlg, IDC_WIOBIT2) == BST_CHECKED) ? 4: 0;
				byteToSend |= (IsDlgButtonChecked(hDlg, IDC_WIOBIT3) == BST_CHECKED) ? 8: 0;
				byteToSend |= (IsDlgButtonChecked(hDlg, IDC_WIOBIT4) == BST_CHECKED) ? 16: 0;
				byteToSend |= (IsDlgButtonChecked(hDlg, IDC_WIOBIT5) == BST_CHECKED) ? 32: 0;
				byteToSend |= (IsDlgButtonChecked(hDlg, IDC_WIOBIT6) == BST_CHECKED) ? 64: 0;
				byteToSend |= (IsDlgButtonChecked(hDlg, IDC_WIOBIT7) == BST_CHECKED) ? 128: 0;

				if(writeByte(byteToSend) < 0) {
					tryReestablishe(hDlg);
				}
			} else if(IsDlgButtonChecked(hDlg, IDC_WLN) == BST_CHECKED) {
				byteToSend |= (IsDlgButtonChecked(hDlg, IDC_WIOBIT0) == BST_CHECKED) ? 1: 0;
				byteToSend |= (IsDlgButtonChecked(hDlg, IDC_WIOBIT1) == BST_CHECKED) ? 2: 0;
				byteToSend |= (IsDlgButtonChecked(hDlg, IDC_WIOBIT2) == BST_CHECKED) ? 4: 0;
				byteToSend |= (IsDlgButtonChecked(hDlg, IDC_WIOBIT3) == BST_CHECKED) ? 8: 0;

				if(writeLowNibble(byteToSend) < 0) {
					tryReestablishe(hDlg);
				}

			} else if(IsDlgButtonChecked(hDlg, IDC_WHN) == BST_CHECKED) {
				int highNibbleToSend = 0;
				highNibbleToSend |= (IsDlgButtonChecked(hDlg, IDC_WIOBIT4) == BST_CHECKED) ? 1: 0;
				highNibbleToSend |= (IsDlgButtonChecked(hDlg, IDC_WIOBIT5) == BST_CHECKED) ? 2: 0;
				highNibbleToSend |= (IsDlgButtonChecked(hDlg, IDC_WIOBIT6) == BST_CHECKED) ? 4: 0;
				highNibbleToSend |= (IsDlgButtonChecked(hDlg, IDC_WIOBIT7) == BST_CHECKED) ? 8: 0;

				if(writeHighNibble(highNibbleToSend) < 0) {
					tryReestablishe(hDlg);
				}
			} else if(IsDlgButtonChecked(hDlg, IDC_WCT) == BST_CHECKED) {
				int ctrlToSend = 0;
				ctrlToSend |= (IsDlgButtonChecked(hDlg, IDC_WCTBIT0) == BST_CHECKED) ? 1: 0;
				ctrlToSend |= (IsDlgButtonChecked(hDlg, IDC_WCTBIT1) == BST_CHECKED) ? 2: 0;

				if(writeCtrl(ctrlToSend) < 0) {
					tryReestablishe(hDlg);
				}
			}
		}
		break;
	}
	return (INT_PTR)FALSE;
}

INT_PTR CALLBACK About(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) {
	UNREFERENCED_PARAMETER(lParam);
	switch (message)
	{
	case WM_INITDIALOG:			
		return (INT_PTR)TRUE;
	case WM_COMMAND:
		if (LOWORD(wParam) == IDOK) {
			EndDialog(hDlg, LOWORD(wParam));
			return (INT_PTR)TRUE;
		}
		break;
	}
	return (INT_PTR)FALSE;
}

void tryReestablishe(HWND hwnd) {
	if(init16FUSB() == -1) {
		MessageBox(hwnd, L"Can't communicate with device!", L"Error", NULL);
	} else {
		MessageBox(hwnd, L"The communication with device was reestablished!", L"Communication reestablished", NULL);
	}
}