/* ---------------------------------------------------------------------------
** This software is part of 16FUSB project.
**
** 16FUSB_Dio.cpp
** Functions to comunicate with 16FUSB via libusb.
**
** Author: Emanuel Paz <efspaz@gmail.com>
** -------------------------------------------------------------------------*/
#include "16FUSB_Dio.h"

usb_dev_handle *h16FUsb;

int init16FUSB() {

	struct usb_bus *bus;
    struct usb_device *dev;

	usb_init();			 // initialize the library
    usb_find_busses();   // find all busses
    usb_find_devices();  // find all connected devices

	for (bus = usb_get_busses(); bus; bus = bus->next)
    {
        for (dev = bus->devices; dev; dev = dev->next)
        {
            if (dev->descriptor.idVendor == _16FUSB_VID && dev->descriptor.idProduct == _16FUSB_PID)
            {
                h16FUsb = usb_open(dev);
				return 0;
            }
        }
    }
	return -1;
}

void close16FUSB() {
	usb_close(h16FUsb);
}

int writeByte(int byteToSend) {
	return usb_control_msg(
				h16FUsb,
				0x40,
				0x01,
				0x0001,			// WriteByte -> wValue = 0x0001
				byteToSend,				
				NULL,
				0,
				1000
			);
}

int writeLowNibble(int nibbleToSend) {
	return usb_control_msg(
				h16FUsb,
				0x40,
				0x01,
				0x0002,			// WriteLowNibble -> wValue = 0x0002
				nibbleToSend,
				NULL,
				0,
				1000
			);
}

int writeHighNibble(int nibbleToSend) {
	return usb_control_msg(
				h16FUsb,
				0x40,
				0x01,
				0x0004,			// WriteHighNibble -> wValue = 0x0004
				nibbleToSend,
				NULL,
				0,
				1000
			);
}

int writeCtrl(int byteToSend) {
		return usb_control_msg(
				h16FUsb,
				0x40,
				0x01,
				0x0008,			// WriteCtrl -> wValue = 0x0008
				byteToSend,
				NULL,
				0,
				1000
			);
}

char readByte(char buffer[]) {
	int ret = 0;
	ret = usb_control_msg(
		h16FUsb,
		0xC0,
		0x01,
		0x0001,				// ReadByte -> wValue = 0x0001
		0,
		buffer,				// Buffer to receive data from host
		1,					// just one byte
		1000
	);
	return ret;	
}

char readLowNibble(char buffer[]) {
	int ret = 0;
	ret = usb_control_msg(
		h16FUsb,
		0xC0,
		0x01,
		0x0002,				// ReadLowNibble -> wValue = 0x0002
		0,
		buffer,				// Buffer to receive data from host
		1,					// just one byte
		1000
	);
	return ret;
}

char readHighNibble(char buffer[]) {
	int ret = 0;
	ret = usb_control_msg(
		h16FUsb,
		0xC0,
		0x01,
		0x0002,				// ReadHighNibble -> wValue = 0x0004
		0,
		buffer,				// Buffer to receive data from host
		1,					// just one byte
		1000
	);
	return ret;
}

char readCtrl(char buffer[]) {
	int ret = 0;
	ret = usb_control_msg(
		h16FUsb,
		0xC0,
		0x01,
		0x0008,				// ReadCtrl -> wValue = 0x0008
		0,
		buffer,				// Buffer to receive data from host
		1,					// just one byte
		1000
	);
	return ret;
}

char readStatus(char buffer[]) {
	int ret = 0;
	ret = usb_control_msg(
		h16FUsb,
		0xC0,
		0x01,
		0x0010,				// ReadStatus -> wValue = 0x0010
		0,
		buffer,				// Buffer to receive data from host
		1,					// just one byte
		1000
	);
	return ret;
}
